#!/bin/sh

#
# tla.sh, estula.sh, tou.sh - affiche la machine sur laquelle est
# connecte un utilisateur
#
# Usage :
# tla.sh <logname> [-t <timeout>] [-d <delay>]
#
# estula.sh <logname> [-t <timeout>] [-d <delay>]
#
# tou.sh <logname> [-t <timeout>] [-d <delay>]
#
# Description :
# Chacun de ces programmes execute sur un ensemble de
# machines definies dans le fichier [~/.diffusion] une commande cherchant
# l'utilisateur <logname>. Si la personne est connectee, un mail est envoye
# a l'utilisateur dont le nom est contenu dans la variable d'environnement
# LOGNAME et le programme se termine. Sinon une autre verification est
# a nouveau effectuee apres un delai de <delay> (60 secondes par defaut).
# Une valeur de timeout peut etre passee au programme pour donner le temps
# de repondre aux serveurs distant. Par defaut cette valeur est de 3 secondes.
#
# Programmes :
# Les programmes cherchant un utilisateur sont les suivants :
# 	tla.sh : finger [logname]@host
# 	estula.sh : rsh host finger [logname]
# 	tou.sh : rsh ps aux | grep [logname]
#
# Historiquement, j'ai ecrit d'abord tla.sh, qui ne marchait pas si le
# port de finger etait ferme, ce qui est habituellement le cas pour des
# raisons de securite. Ensuite j'ai ecrit estula.sh, qui est utilise rsh,
# mais si l'utilisateur a un fichier .nofinger cette commande ne fonctionne
# pas. La commande tou.sh regle le probleme en cherchant les processus
# lances par l'utilisateur recherche.
#
# Fichiers :
# ~/.diffusion : contient la liste des machines sur lesquelles il faut
#  chercher l'utilisateur
#
# Requiere :
# finger, rsh, mail
#
# Variables :
# 	LOGNAME : nom de l'utilisateur a qui il faut envoyer par mel le
# resultat de la commande
#

[ $# -gt 5 -o $# -lt 1 ] && echo >&2 "Usage : `basename $0` <logname> [-t <timeout>] [-d <delay>]" && exit 1

while [ -n "$1" ]
do
    case "$1" in
        -t) TimeOut="$2" ; shift ;;
        -d) Delay="$2" ; shift ;;
        *)  Nom="$1" ;;
    esac
    shift
done

#[ $# -eq 2 ] && TimeOut=$2 || TimeOut=3

#exec 2> /dev/null

LOGTO=$LOGNAME

[ -z "$Delay" ] && Delay=60
[ -z "$TimeOut" ] && TimeOut=3

trap 'rm -f ~/.estula$$ /tmp/estula.$$ ; exit 0' INT ABRT KILL

echo 0 > ~/.estula$$  
AncienneTaille=0

while true
do
    for machine in `cat ~/.diffusion`
    do
        [ -f ~/.estula$$ ] && [ `cat ~/.estula$$` -eq 1 ] && exit 0
        (
        rsh $machine "finger | grep \"^$Nom\"" >> /tmp/estula.$$
        ) &
        pid=$!
        sleep $TimeOut
        retour=`cat /tmp/estula.$$`
        kill $pid
        NouvelleTaille=`ls -l /tmp/estula.$$ | awk '{print $5}'`
        [ $NouvelleTaille -ne $AncienneTaille ] && echo "$machine" >> /tmp/estula.$$ && AncienneTaille=`ls -l /tmp/estula.$$ | awk '{print $5}'`
        if [ -n "$retour" -a `cat ~/.estula$$` -eq 0 ]
        then
                echo 1 > ~/.estula$$
                set -- `grep $Nom /tmp/estula.$$`
                Machine=`tail -n 1 /tmp/estula.$$`
                mail -s "[estula] $2 $3 logu" $LOGTO << FIN
Vite ! $2 $3 vient de se loguer sur $Machine !

`date`

`sed "/$Machine/d" < /tmp/estula.$$`
FIN
        fi
    done

    sleep $Delay
done

rm -f ~/.estula$$ /tmp/estula.$$

exit 0
